%module matrix

%{
#define SWIG_FILE_WITH_INIT
#include "libharkio3.h"
#include "libharkio3_macros.h"
#define HARKIO_LOG_INITIAL_LEVEL harkio_Log_Error
%}
%include <complex.i>


typedef struct{
    char tag[32];        /*!< 保存時に埋め込まれる */
    char type[32];       /*!< データ型。 */
    int  dim;            /*!< 次元 */
    int  rows;           /*!< 行サイズ */
    int  cols;           /*!< 列サイズ */
    multipleType data;   /*!< データ自体 */
} harkio_Matrix;

%extend harkio_Matrix {
    harkio_Matrix(char *type, int col, int row) {
        return harkio_Matrix_new(type, col, row);
    }
    harkio_Matrix(char *filename) {
        return harkio_Matrix_fromFile(filename);
    }
    ~harkio_Matrix() {
    }
 
    char *__str__(){
        return harkio_Matrix_printBuffer($self);
    }
    harkio_Matrix* __add__(harkio_Matrix *other) {
        return harkio_Matrix_add($self, other);
    }
    harkio_Matrix* __sub__(harkio_Matrix *other) {
        return harkio_Matrix_sub($self, other);
    }
    harkio_Matrix* __mul__(harkio_Matrix *other) {
        return harkio_Matrix_mul($self, other);
    }
    //harkio_Matrix* __getitem__(int* pos){
//int harkio_Matrix_getValueInt32(harkio_Matrix *m, int row, int col, int *value);
//int harkio_Matrix_getValueFloat32(harkio_Matrix *m, int row, int col, float *value);
//int harkio_Matrix_getValueComplex(harkio_Matrix *m, int row, int col, COMPLEX_TYPE *value);
//harkio_Matrix* harkio_Matrix_getValues(harkio_Matrix *matrix,int rowFrom ,int rowTo,int colFrom,int colTo);
//   }
    int __setitem__(PyObject *pos, int value){
        int row = PyLong_AsLong(PyTuple_GetItem(pos, 0));
        int col = PyLong_AsLong(PyTuple_GetItem(pos, 1));
        if(strncmp($self->type, "int32", 5) == 0){
            return harkio_Matrix_setValueInt32($self, row, col, value);
        } else {
            return -1;
        }
    }
    int __setitem__(PyObject *pos, float value){
        int row = PyLong_AsLong(PyTuple_GetItem(pos, 0));
        int col = PyLong_AsLong(PyTuple_GetItem(pos, 1));
        if(strncmp($self->type, "float32", 5) == 0){
            return harkio_Matrix_setValueFloat32($self, row, col, value);
        } else {
            return -1;
        }
    }
    int __setitem__(PyObject *pos, COMPLEX_TYPE value){
        int row = PyLong_AsLong(PyTuple_GetItem(pos, 0));
        int col = PyLong_AsLong(PyTuple_GetItem(pos, 1));
        if(strncmp($self->type, "complex", 7) == 0){
            // COMPLEX_TYPE v;
            //  = value->real + _Complex_I * value->imag;
            return harkio_Matrix_setValueComplex($self, row, col, value);
        } else {
            return -1;
        }
    }
    int harkio_Matrix_write(char *filename) {
        return harkio_Matrix_write($self, filename);
    }
};


harkio_Matrix* harkio_Matrix_eye(char *type, int size);
harkio_Matrix* harkio_Matrix_zeros(char *type, int rows, int cols);
harkio_Matrix* harkio_Matrix_ones(char *type, int rows, int cols);
harkio_Matrix* harkio_Matrix_fromZip(struct zip *handle, char *path);
harkio_Matrix* harkio_Matrix_copy(harkio_Matrix *m);

char* harkio_Matrix_writeBuffer(harkio_Matrix *m, size_t *size);
size_t harkio_Matrix_size(harkio_Matrix *m, int *size);

harkio_Matrix* harkio_Matrix_emul(harkio_Matrix *a, harkio_Matrix *b);
harkio_Matrix* harkio_Matrix_sumRow( harkio_Matrix *matrix);
harkio_Matrix* harkio_Matrix_sumCol( harkio_Matrix *matrix);
harkio_Matrix* harkio_Matrix_sumAll( harkio_Matrix *matrix);

harkio_Matrix* harkio_Matrix_sum( harkio_Matrix **matrix , int num );

harkio_Matrix* harkio_Matrix_ediv(harkio_Matrix *a, harkio_Matrix* b);
harkio_Matrix* harkio_Matrix_getDiag(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_makeDiag(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_esquare(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_eabs(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_econj(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_etanh(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_ecosh(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_elog(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_elog10(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_ereal(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_eimag(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_emulCInt32(harkio_Matrix *m, int value);
harkio_Matrix* harkio_Matrix_emulCFloat32(harkio_Matrix *m, float value);
harkio_Matrix* harkio_Matrix_emulCComplex(harkio_Matrix *m, COMPLEX_TYPE value);
harkio_Matrix* harkio_Matrix_edivCInt32(harkio_Matrix *m, int value);
harkio_Matrix* harkio_Matrix_edivCFloat32(harkio_Matrix *m, float value);
harkio_Matrix* harkio_Matrix_edivCComplex(harkio_Matrix *m, COMPLEX_TYPE value);
harkio_Matrix* harkio_Matrix_epowerCInt32(harkio_Matrix *m, int value);
harkio_Matrix* harkio_Matrix_epowerCFloat32(harkio_Matrix *m, float value);
harkio_Matrix* harkio_Matrix_epowerCComplex(harkio_Matrix *m, COMPLEX_TYPE value);
harkio_Matrix* harkio_Matrix_emap(harkio_Matrix *m, harkio_Matrix_maps *map);
harkio_Matrix* harkio_Matrix_ecalcCInt32(harkio_Matrix *m, int value, harkio_Matrix_calcfuncInt32 func);
harkio_Matrix* harkio_Matrix_ecalcCFloat32(harkio_Matrix *m, float value, harkio_Matrix_calcfuncFloat32 func);
harkio_Matrix* harkio_Matrix_ecalcCComplex(harkio_Matrix *m, COMPLEX_TYPE value, harkio_Matrix_calcfuncComplex func);
harkio_Matrix* harkio_Matrix_reshape(harkio_Matrix *m, int rows, int cols);
harkio_Matrix* harkio_Matrix_transpose(harkio_Matrix *m);
harkio_Matrix* harkio_Matrix_ctranspose(harkio_Matrix *m);

//int harkio_Matrix_setValues( harkio_Matrix *m1 , harkio_Matrix *m2 , int row , int col );
harkio_Matrix*  harkio_Matrix_getValues(harkio_Matrix *matrix,int rowFrom ,int rowTo,int colFrom,int colTo);
char* harkio_Matrix_getTag(harkio_Matrix *xml);
int harkio_Matrix_isEqual(harkio_Matrix *a, harkio_Matrix *b);
int harkio_Complex_calc(COMPLEX_TYPE *value , COMPLEX_TYPE valuea , COMPLEX_TYPE valueb , enum Calculate calc);
int harkio_CheckType(char *type);
